package org.asciidoc.intellij.psi;

import com.intellij.psi.impl.source.tree.FileElement;
import org.asciidoc.intellij.lexer.AsciiDocTokenTypes;
import org.jetbrains.annotations.NotNull;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class AsciiDocSectionTest {

  private static class MockTitleAsciiDocSection extends AsciiDocSectionImpl {
    private final String title;

    MockTitleAsciiDocSection(String title) {
      super(new FileElement(AsciiDocTokenTypes.WHITE_SPACE, ""));
      this.title = title;
    }

    @Override
    protected String getAttribute(String attr, String varDefault) {
      return "_";
    }

    @NotNull
    @Override
    public String getTitle() {
      return title;
    }

    @Override
    public @NotNull String getTitleNoSubstitution() {
      return title;
    }
  }

  @Test
  public void testIdContainsSeparators() {
    MockTitleAsciiDocSection section = new MockTitleAsciiDocSection("My Title.");
    assertEquals("_my_title", section.getAutogeneratedId());
  }

  @Test
  public void testIdContainsUtf8Characters() {
    // note the the "soft i" in this czech headline
    MockTitleAsciiDocSection section = new MockTitleAsciiDocSection("Nadpis\u00EDk");
    assertEquals("_nadpis\u00EDk", section.getAutogeneratedId());
  }

  @Test
  public void testMatchingIdIgnoresSuffix() {
    MockTitleAsciiDocSection section = new MockTitleAsciiDocSection("My Title.");
    assertTrue(section.matchesAutogeneratedId("_my_title_2"));
  }
}
